package constants

import "time"

const (
	ConsensusAccountName = "consensus"
	ProtoConfigPath      = "protocol"

	// MaxAlphabetNodes is the maximum number of candidates allowed, which is currently limited by the size
	// of the invocation script.
	// See: https://github.com/nspcc-dev/neo-go/blob/740488f7f35e367eaa99a71c0a609c315fe2b0fc/pkg/core/transaction/witness.go#L10
	MaxAlphabetNodes = 22

	SingleAccountName    = "single"
	CommitteeAccountName = "committee"

	NNSContract        = "nns"
	FrostfsContract    = "frostfs"    // not deployed in side-chain.
	ProcessingContract = "processing" // not deployed in side-chain.
	AlphabetContract   = "alphabet"
	BalanceContract    = "balance"
	ContainerContract  = "container"
	FrostfsIDContract  = "frostfsid"
	NetmapContract     = "netmap"
	PolicyContract     = "policy"
	ProxyContract      = "proxy"

	ContractWalletFilename    = "contract.json"
	ContractWalletPasswordKey = "contract"

	FrostfsOpsEmail  = "ops@frostfs.info"
	NNSRefreshDefVal = int64(3600)
	NNSRetryDefVal   = int64(600)
	NNSTtlDefVal     = int64(3600)

	DefaultExpirationTime = 10 * 365 * 24 * time.Hour / time.Second

	DeployMethodName = "deploy"
	UpdateMethodName = "update"

	TestContractPassword = "grouppass"
)

var (
	ContractList = []string{
		BalanceContract,
		ContainerContract,
		FrostfsIDContract,
		NetmapContract,
		PolicyContract,
		ProxyContract,
	}

	FullContractList = append([]string{
		FrostfsContract,
		ProcessingContract,
		NNSContract,
		AlphabetContract,
	}, ContractList...)
)
