package netmap

import (
	"fmt"
	"strings"

	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-adm/internal/modules/morph/constants"
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-adm/internal/modules/morph/helper"
	"github.com/nspcc-dev/neo-go/pkg/io"
	"github.com/nspcc-dev/neo-go/pkg/rpcclient/management"
	"github.com/spf13/cobra"
	"github.com/spf13/viper"
)

func ForceNewEpochCmd(cmd *cobra.Command, _ []string) error {
	wCtx, err := helper.NewInitializeContext(cmd, viper.GetViper())
	if err != nil {
		return fmt.Errorf("can't to initialize context: %w", err)
	}

	r := management.NewReader(wCtx.ReadOnlyInvoker)
	cs, err := r.GetContractByID(1)
	if err != nil {
		return fmt.Errorf("can't get NNS contract info: %w", err)
	}

	nmHash, err := helper.NNSResolveHash(wCtx.ReadOnlyInvoker, cs.Hash, helper.DomainOf(constants.NetmapContract))
	if err != nil {
		return fmt.Errorf("can't get netmap contract hash: %w", err)
	}

	bw := io.NewBufBinWriter()
	if err := helper.EmitNewEpochCall(bw, wCtx, nmHash); err != nil {
		return err
	}

	if err = wCtx.SendConsensusTx(bw.Bytes()); err == nil {
		err = wCtx.AwaitTx()
	}
	if err != nil && strings.Contains(err.Error(), "invalid epoch") {
		cmd.Println("Epoch has already ticked.")
		return nil
	}
	return err
}
